from config import config
from nanpy.arduinotree import ArduinoTree
from nanpy.serialmanager import serial_manager


def soft_reset():
    serial_manager.sleep_after_connect = config['sleep_after_connect']
    ArduinoTree().soft_reset()


def setup():
    soft_reset()



from nanpy.arduinoboard import arduinomethod, returns, FirmwareClass
from nanpy.classinfo import check4firmware


@check4firmware
class ArduinoApi(FirmwareClass):

    """access to Arduino public API functions."""
    firmware_id = 'A'

    LOW, HIGH = 0, 1
    INPUT, OUTPUT = 0, 1
    LSBFIRST, MSBFIRST = 0, 1

    @arduinomethod('dw')
    def digitalWrite(self, pin, value):
        pass

    @returns(int)
    @arduinomethod('r')
    def digitalRead(self, pin):
        pass

    @arduinomethod('aw')
    def analogWrite(self, pin, value):
        pass

    @returns(int)
    @arduinomethod('a')
    def analogRead(self, pin):
        pass

    @arduinomethod('pm')
    def pinMode(self, pin, mode):
        pass

    @returns(int)
    @arduinomethod('m')
    def millis(self):
        pass

    @returns(int)
    @arduinomethod('s')
    def shiftOut(self, dataPin, clockPin, bitOrder, value):
        """Shifts out a byte of data one bit at a time.

        :param dataPin: the pin on which to output each bit (int)
        :param clockPin: the pin to toggle once the dataPin has been set to the correct value (int)
        :param bitOrder: which order to shift out the bits; either MSBFIRST or LSBFIRST.
        :param value: the data to shift out. (byte)

        http://arduino.cc/en/Reference/shiftOut

        """



from nanpy.arduinoboard import arduinomethod, FirmwareClass
from nanpy.classinfo import check4firmware


@check4firmware
class Watchdog(FirmwareClass):

    """Direct access to watchdog timer.

    Details: http://www.nongnu.org/avr-libc/user-manual/group__avr__watchdog.html

    """
    WDTO_15MS = 0
    WDTO_30MS = 1
    WDTO_60MS = 2
    WDTO_120MS = 3
    WDTO_250MS = 4
    WDTO_500MS = 5
    WDTO_1S = 6
    WDTO_2S = 7
    WDTO_4S = 8
    WDTO_8S = 9

    @arduinomethod
    def reset(self):
        """Reset the watchdog timer. (wdt_reset)

        When the watchdog timer is enabled,
        a call to this instruction is required before the timer expires,
        otherwise a watchdog-initiated device reset will occur.

        """

    @arduinomethod
    def enable(self, timeout):
        """Enable the watchdog timer, configuring it for expiry after
        timeout. (wdt_enable)"""

    @arduinomethod
    def disable(self):
        """Disable the watchdog timer, if possible. (wdt_disable)"""



# Nanpy
# Copyright 2012 Andrea Stagi
# See LICENSE for details.

"""
Nanpy library
"""
__version__ = '1.0'
__author__ = 'Andrea Stagi'
__license__ = 'MIT'

from nanpy.arduinoboard import ArduinoObject
from nanpy.serialmanager import SerialManager
from nanpy.serialmanager import serial_manager

from nanpy.arduino import Arduino
from nanpy.eeprom import EEPROM
from nanpy.lcd import Lcd
from nanpy.onewire import OneWire
from nanpy.stepper import Stepper
from nanpy.servo import Servo
from nanpy.dallastemperature import DallasTemperature
from nanpy.tone import Tone
from nanpy.capacitivesensor import CapacitiveSensor
from nanpy.dht import DHT
from nanpy.arduinoapi import ArduinoApi
from nanpy.eepromobj import EepromLib



from nose.tools import eq_, ok_, eq_, ok_
from tests.util import exc_
from nanpy.arduinotree import ArduinoTree
import common


def setup():
    common.setup()


def test_registers():
    a = ArduinoTree()
    print (a.register.names)
    ok_(len(a.register.names) > 77)

    eq_(a.register.get('DDRB').read_value(), 0)
    eq_(a.register.get('DDRB').value, 0)
    exc_(ValueError, lambda x: a.register.get('xxx').value, 0)

    a.register.get('DDRB').write_value(5)
    eq_(a.register.get('DDRB').value, 5)
    a.register.get('DDRB').value = 3
    eq_(a.register.get('DDRB').value, 3)
    a.register.get('DDRB').write_value(0)

    eq_(a.register.get('DDRB').address, 0x24)
    eq_(a.register.get('DDRB').name, 'DDRB')
    eq_(a.register.get('DDRB').size, 1)

    # 9 bit
    eq_(a.register.get('EEAR').size, 2)
    a.register.get('EEAR').value = 511
    eq_(a.register.get('EEAR').value, 511)


def test_register_proxy():
    a = ArduinoTree()
    r = a.register.proxy

    eq_(r.DDRB, 0)

    r.DDRB = 5
    eq_(r.DDRB, 5)
    r.DDRB = 3
    eq_(r.DDRB, 3)
    eq_(a.register.get('DDRB').value, 3)

    r.DDRB = 0

    # 9 bit
    r.EEAR = 511
    eq_(r.EEAR, 511)



from nanpy.memo import memoized
import fnmatch
import logging
import serial
import sys
import time

DEFAULT_BAUDRATE = 115200

log = logging.getLogger(__name__)

PY3 = sys.version_info[0] == 3


class SerialManagerError(Exception):
    pass


def _auto_detect_serial_unix(preferred_list=['*']):
    import glob
    glist = glob.glob('/dev/ttyUSB*') + glob.glob('/dev/ttyACM*')
    glist += ['/dev/ttyS0', '/dev/ttyS1']
    ret = []

    for d in glist:
        for preferred in preferred_list:
            if fnmatch.fnmatch(d, preferred):
                ret.append(d)
    if len(ret) > 0:
        return ret

    for d in glist:
        ret.append(d)
    return ret


class NoneSerialManager(object):

    def write(self, val):
        pass

    def read(self):
        return ""

    def readline(self):
        return ""


class SerialManager(object):
    _serial = None

    def __init__(self, device=None,
                 baudrate=DEFAULT_BAUDRATE,
                 sleep_after_connect=2,
                 timeout=1):
        self.device = device
        self.baudrate = baudrate
        self.sleep_after_connect = sleep_after_connect
        self.timeout = timeout

    def open(self, device=None):
        '''open connection'''
        if device:
            self.device = device

        if not self.device:
            ports = _auto_detect_serial_unix()
            if not len(ports):
                raise SerialManagerError("No port was set, and no port was found!")
            self.device = ports[0]
        log.debug('opening port:%s [%s baud]' % (self.device, self.baudrate))
        assert self.device
        self._serial = serial.Serial(self.device,
                                     self.baudrate,
                                     timeout=self.timeout)
        if self.sleep_after_connect:
            time.sleep(self.sleep_after_connect)
        self._serial.flushInput()

    def write(self, value):
        if not self._serial:
            self.open()
        log.debug('sending:%s' % repr(value))
        if PY3:
            self._serial.write(bytes(value, 'latin-1'))
        else:
            self._serial.write(value)

    def readline(self):
        if not self._serial:
            self.open()
        s = self._serial.readline()
        log.debug('received:%s' % repr(s))
        s = s.decode()
        if not len(s):
            raise SerialManagerError('Serial timeout!')
        return s

    def flush_input(self):
        '''Flush input buffer, discarding all it's contents.'''
        if not self._serial:
            self.open()
        self._serial.flushInput()

    def close(self):
        '''close connection'''
        if self._serial:
            self._serial.close()
            self._serial = None

serial_manager = SerialManager()




from nanpy.examples.boottime import boot_time
from nanpy.examples.checkspeed import checkspeed
from nanpy.examples.counterdemo import counterdemo
from nanpy.examples.dds import dds
from nanpy.examples.dump import dumpall
from nanpy.examples.highfreqpwm import highfreqpwm
from nanpy.examples.ramdump import ramdump
from nanpy.examples.reset import reset_demo
import common


def setup():
    common.setup()


def test_dump():
    dumpall()


def test_boot_time():
    boot_time()


def test_reset_demo():
    reset_demo()


def test_highfreqpwm():
    highfreqpwm()


def test_counter():
    counterdemo()


def test_dds():
    dds()


def test_checkspeed():
    checkspeed(n=1)


def test_ramdump():
    ramdump()



from nanpy.arduinoboard import ArduinoObject
from nanpy.arduinoboard import (arduinoobjectmethod, returns)

class Servo(ArduinoObject):

    def __init__(self, pin):
        ArduinoObject.__init__(self)
        self.id = self.call('new', pin)

    @arduinoobjectmethod
    def write(self, value):
        pass

    @returns(int)
    @arduinoobjectmethod
    def read(self):
        pass

    @arduinoobjectmethod
    def writeMicroseconds(self, value):
        pass

    @returns(int)
    @arduinoobjectmethod
    def readMicroseconds(self, value):
        pass

    @arduinoobjectmethod
    def attached(self):
        pass



from nanpy.arduinotree import ArduinoTree
import time
from nanpy.serialmanager import SerialManager


GATE_TIME = 0.7  # sec
F = 9765  # Hz


def counterdemo():
    connection = SerialManager()
    a = ArduinoTree(connection=connection)
    p = a.pin.get(5)
    a.soft_reset()
    print ('PWM frequencies_available: %s' % p.pwm.frequencies_available)
    p.write_mode(1)
    p.pwm.write_value(128)
    p.pwm.frequency = F
    print ('frequency set: %s' % p.pwm.frequency)
    t1 = time.time()
    fread = a.counter.read_frequency(GATE_TIME)
    t2 = time.time()
    print ('frequency read: %s' % fread)
    print ('gate time: %s sec' % (GATE_TIME))
    print ('time elapsed: %s sec' % (t2 - t1))

if __name__ == '__main__':
    counterdemo()



from config import config
from nose.tools import eq_
from nanpy.arduinotree import ArduinoTree
import common


def setup():
    common.setup()


INPUT, OUTPUT = 0, 1
F_CPU = config.get('F_CPU')


def test_pwm():
    a = ArduinoTree()
    pin8 = a.pin.get(8)
    pin9 = a.pin.get(9)

    TCCR1B = a.register.get('TCCR1B')

    eq_(pin8.pwm.available, False)
    eq_(pin9.pwm.available, True)
    eq_(pin9.pwm.timer_register_name_b, 'TCCR1B')
    eq_(pin9.pwm.base_divisor, 512)
    eq_(pin9.pwm.divisors_available, [1, 8, 64, 256, 1024])

    frequencies = sorted([F_CPU / 2 ** 9,
                          F_CPU / 2 ** 12,
                          F_CPU / 2 ** 15,
                          F_CPU / 2 ** 17,
                          F_CPU / 2 ** 19,
                          ])
    eq_(pin9.pwm.frequencies_available, frequencies)

    TCCR1B.value = 3
    eq_(TCCR1B.value, 3)

    eq_(pin9.pwm.read_divisor(), 2 ** 6)
    eq_(pin9.pwm.divisor, 2 ** 6)
    eq_(pin9.pwm.read_frequency(), F_CPU / 2 ** 15)
    eq_(pin9.pwm.frequency, F_CPU / 2 ** 15)

    pin9.pwm.write_frequency(int(F_CPU / 2 ** 19))
    eq_(pin9.pwm.read_frequency(), F_CPU / 2 ** 19)
    eq_(TCCR1B.value, 5)

    TCCR1B.value = 2
    eq_(pin9.pwm.read_divisor(), 2 ** 3)
    eq_(pin9.pwm.divisor, 2 ** 3)
    eq_(pin9.pwm.read_frequency(), F_CPU / 2 ** 12)
    eq_(pin9.pwm.frequency, F_CPU / 2 ** 12)
    eq_(TCCR1B.value, 2)

    pin9.pwm.divisor = 2 ** 10
    eq_(pin9.pwm.read_divisor(), 2 ** 10)
    eq_(pin9.pwm.divisor, 2 ** 10)
    eq_(pin9.pwm.read_frequency(), F_CPU / 2 ** 19)
    eq_(pin9.pwm.frequency, F_CPU / 2 ** 19)
    eq_(TCCR1B.value, 5)

    pin9.pwm.write_frequency(F_CPU / 2 ** 9)
    eq_(pin9.pwm.read_frequency(), F_CPU / 2 ** 9)
    eq_(TCCR1B.value, 1)

    pin9.pwm.frequency = F_CPU / 2 ** 17
    eq_(pin9.pwm.read_frequency(), F_CPU / 2 ** 17)
    eq_(TCCR1B.value, 4)

    TCCR1B.value = 3
    eq_(TCCR1B.value, 3)

    pin9.pwm.write_value(45)



#!/usr/bin/env python

from nanpy import SerialManager
from nanpy.lcd import Lcd
from nanpy.lcd_i2c import Lcd_I2C

I2C = True


def hello():
    connection = SerialManager(sleep_after_connect=2)

    cols, rows = 16, 2

    if I2C:
        pins = [0x27, 2, 1, 0, 4, 5, 6, 7, 3, 0]  # "ebay" version
        lcd = Lcd_I2C(pins, [cols, rows], connection=connection)
        lcd.setBacklight(0)
    else:
        pins = [7, 8, 9, 10, 11, 12]
        lcd = Lcd(pins, [cols, rows], connection=connection)

    lcd.setCursor(0, 0)
    lcd.printString('hello')

if __name__ == '__main__':
    hello()



from nanpy.memo import memoized
from nanpy.pwm import ArduinoPwmPin
import sys

LOW, HIGH = 0, 1
INPUT, OUTPUT = 0, 1

# from six
PY3 = sys.version_info[0] == 3
if PY3:
    string_types = str,
else:
    string_types = basestring,


class PinError(Exception):
    pass


def to_pin_number(pin_name, A0=None):
    try:
        if isinstance(pin_name, string_types):
            if pin_name[0] == 'D':
                nr = int(pin_name[1:])
                if A0 is not None:
                    if nr >= A0:
                        raise ValueError('invalid pin id:%r' % pin_name)
            elif pin_name[0] == 'A':
                if A0 is None:
                    raise ValueError('A0 is None, but analog pin was set! (%s)' % pin_name)
                nr = int(pin_name[1:]) + A0
            else:
                nr = int(pin_name)
        elif hasattr(pin_name, 'pin_number'):
            nr=pin_name.pin_number
        else:
            nr = int(pin_name)
    except IndexError:
        raise ValueError('invalid pin id:%r' % pin_name)
    return nr


class ArduinoPin(object):

    '''Object-oriented representation of an Arduino pin

    Examples:

        # they do the same
        a=ArduinoTree()
        a.pin.get(0).value
        a.pin.get('D0').value
        a.pin.get('D0').read_value()
    '''

    def __init__(self, name, total_pin_count, define, register, core, ram, api):
        """name can be int or string."""
        self.register = register
        self.core = core
        self.ram = ram
        self.api = api
        self.define = define
        self.A0 = define.get('A0')
        self.pin_number = to_pin_number(name, self.A0)
        if self.pin_number >= total_pin_count:
            raise ValueError('pin %s (Nr:%s) not in range' %
                             (name, self.pin_number))

    @property
    @memoized
    def pwm(self):
        '''Object-oriented representation of the pin PWM functionality
        '''
        return (
            ArduinoPwmPin(
                self.pin_number,
                self.define,
                self.register,
                self.core,
                self.api)
        )

    @property
    def is_digital(self):
        return self.pin_number < self.A0

    @property
    def is_analog(self):
        return not self.is_digital

    @property
    def avr_port(self):
        '''AVR port name (example: "B")
        '''
        x = self.core.digitalPinToPort(self.pin_number)
        return chr(ord('A') + x - 1)

    @property
    def avr_bit(self):
        '''AVR bit name (example: "2")
        '''
        bitmask = self.core.digitalPinToBitMask(self.pin_number)
        i = 0
        while bitmask != 1:
            bitmask >>= 1
            i += 1
        return i

    @property
    def avr_pin(self):
        '''AVR pin name (example: "PB2")
        '''
        return 'P%s%s' % (self.avr_port, self.avr_bit)

    @property
    def name(self):
        """Arduino pin name.

        D -> digital
        A -> analog
        (examples: "D2", "A0")

        """
        if self.is_digital:
            return 'D%s' % self.pin_number
        else:
            return 'A%s' % self.pin_number_analog

    @property
    def pin_number_analog(self):
        x = self.pin_number - self.A0
        if x >= 0:
            return x

    @property
    def programming_function(self):
        """programming function (MISO, MOSI, SCK or SS)"""
        if self.pin_number == self.define.get('MISO'):
            return 'MISO'
        if self.pin_number == self.define.get('MOSI'):
            return 'MOSI'
        if self.pin_number == self.define.get('SCK'):
            return 'SCK'
        if self.pin_number == self.define.get('SS'):
            return 'SS'

    def reset(self):
        '''reset to the pin default state: INPUT, no pullup
        '''
        self.write_mode(INPUT)
        self.write_pullup(LOW)
        if self.pwm.available:
            self.pwm.reset()

    def write_pullup(self, value):
        """set pullup  (0/1)"""
        self.write_mode(INPUT)
        return self.write_digital_value(value)

    def read_digital_value(self, direction=None):
        """read digital value  (0/1)

        direction can be set, if 'direction' parameter exists

        """
        if direction is not None:
            self.write_mode(direction)
        return self.api.digitalRead(self.pin_number)

    def write_digital_value(self, value, direction=None):
        """write digital value  (0/1)

        direction can be set, if 'direction' parameter exists, and it is
        not INPUT

        """
        if direction == INPUT:
            raise ValueError('write_digital_value() to INPUT??')
        if direction is not None:
            self.write_mode(direction)
        value = 1 if value else 0
        return self.api.digitalWrite(self.pin_number, value)

    digital_value = property(read_digital_value, write_digital_value)

    def read_analog_value(self):
        '''read analog value  (0-1023)
        '''
        if not self.is_analog:
            return None
        return self.api.analogRead(self.pin_number)
    analog_value = property(read_analog_value)

    def read_mode(self):
        """read mode  (0/1)"""
        bitmask = self.core.digitalPinToBitMask(self.pin_number)
        port = self.core.digitalPinToPort(self.pin_number)
        reg_address = self.core.portModeRegister(port)
        reg_value = self.ram.read(reg_address)
        mode = OUTPUT if reg_value & bitmask else INPUT
        return mode

    def write_mode(self, value):
        """write mode  (0/1)"""
        return self.api.pinMode(self.pin_number, value)
    mode = property(read_mode, write_mode)


class PinFeature(object):

    def __init__(self, define, register, core, ram, api):
        self.A0 = define.get('A0')
        self.define = define
        self.register = register
        self.core = core
        self.ram = ram
        self.api = api

    @memoized
    def get(self, name):
        return (
            ArduinoPin(
                name,
                self.count,
                self.define,
                self.register,
                self.core,
                self.ram,
                self.api)
        )

    @property
    @memoized
    def count_analog(self):
        """Count of analog pins."""
        return len(self.names_analog)

    @property
    @memoized
    def count_digital(self):
        """Count of digital pins."""
        return len(self.names_digital)

    @property
    @memoized
    def count(self):
        """Count of all pins.
        """
        return self.define.get('NUM_DIGITAL_PINS')

    @property
    def names(self):
        """List of all pin names."""
        return self.names_digital + self.names_analog

    @property
    def names_digital(self):
        """List of digital pin names."""
        A0 = self.A0
        return ['D%s' % x for x in range(0, A0)]

    @property
    def names_analog(self):
        """List of analog pin names."""
        A0 = self.A0
        return (
            ['A%s' % (x - A0) for x in range(A0, self.count)]
        )



from nanpy.ad9850 import AD9850
from nanpy.serialmanager import SerialManager

# http://nr8o.dhlpilotcentral.com/?p=83
W_CLK = 'A5'  # Pin 8 - connect to AD9850 module word load clock pin (CLK)
FQ_UD = 'A4'  # Pin 9 - connect to freq update pin (FQ)
DATA = 'A3'  # Pin 10 - connect to serial data load pin (DATA)
RESET = 'A2'  # Pin 11 - connect to reset pin (RST).

F = 440  # Hz


def dds():
    connection = SerialManager()
    dds = AD9850([W_CLK, FQ_UD, DATA, RESET], connection=connection)

    dds.setup()
    dds.write_frequency(F)


if __name__ == '__main__':
    dds()



from nanpy.arduinoboard import arduinomethod, returns, FirmwareClass
from nanpy.memo import memoized


class FirmwareError(Exception):
    pass


def check4firmware(cls):
    if not hasattr(check4firmware, 'names'):
        check4firmware.names = dict()
    cls_name = cls.get_firmware_id()
    assert cls_name not in check4firmware.names.keys()
    check4firmware.names[cls_name] = cls

    def getinstance(connection):
        if cls_name == 'Info':
            return cls(connection)

        if connection is not None:
            if not hasattr(connection, 'classinfo'):
                connection.classinfo = ClassInfo(connection)
            if cls_name not in connection.classinfo.firmware_id_list:
                raise FirmwareError(
                    '''%s ['%s'] is missing from firmware!''' %
                    (cls, cls_name))
        return cls(connection)
#     getinstance.__name__ = cls.__name__
    return getinstance


@check4firmware
class ClassInfoArray(FirmwareClass):
    firmware_id = 'Info'

    @property
    @memoized
    @returns(int)
    @arduinomethod
    def count(self):
        pass

    @memoized
    @arduinomethod
    def name(self, index):
        pass


class ClassInfo(object):

    """Which classes are compiled into the firmware?"""

    def __init__(self, connection):
        self.firmware_class_status = dict()
        self.unknown_firmware_ids = []

        self._arr = ClassInfoArray(connection=connection)

        ls = [self._arr.name(i) for i in range(self._arr.count)]
        assert len(ls)
        self.firmware_id_list = sorted(ls)

        for cls in check4firmware.names.values():
                self.firmware_class_status[cls.__name__] = False

        for x in self.firmware_id_list:
            cls = check4firmware.names.get(x)
            if cls:
                self.firmware_class_status[cls.__name__] = True
            else:
                self.unknown_firmware_ids.append(x)



#!/usr/bin/env python

# Author: Andrea Stagi <stagi.andrea@gmail.com>
# Description: just a test for Stepper module
# Dependencies: None

from nanpy import Stepper
from nanpy import Arduino

motor = Stepper(100, 9 , 10, 1000)

while True:
    motor.step(-10)
    Arduino.delay(1000)




#!/usr/bin/env python

# Author: Andrea Stagi <stagi.andrea@gmail.com>
# Description: play a melody
# Dependencies: None

from nanpy import Tone

melody = [[Tone.NOTE_C4, 4], [Tone.NOTE_G3, 8], [Tone.NOTE_G3, 8], 
            [Tone.NOTE_A3, 4], [Tone.NOTE_G3, 4], [0, 4], 
            [Tone.NOTE_B3, 4], [Tone.NOTE_C4, 4]]

tone = Tone(4)

for note in melody:
    tone.play(note[0] , 1000/note[1])




from nose.tools import eq_
from nose.tools import ok_
from tests.util import ok_an,  exc_
from nanpy.arduinotree import ArduinoTree
import common


def setup():
    common.setup()


INPUT, OUTPUT = 0, 1


def test_pin_nr():
    a = ArduinoTree()
    pin = a.pin.get(8)

    eq_(pin.pin_number, 8)
    eq_(pin.pin_number_analog, None)

    pin = a.pin.get(14)
    eq_(pin.pin_number, 14)
    eq_(pin.pin_number_analog, 0)

    pin = a.pin.get('A1')
    eq_(pin.pin_number, 15)
    eq_(pin.pin_number_analog, 1)

    pin = a.pin.get('D9')
    eq_(pin.pin_number, 9)
    eq_(pin.pin_number_analog, None)

    pin = a.pin.get('A2')
    eq_(pin.pin_number, 16)
    eq_(pin.pin_number_analog, 2)


def test_is():
    a = ArduinoTree()
    eq_(a.pin.get(8).is_digital, True)
    eq_(a.pin.get(8).is_analog, False)
    eq_(a.pin.get('A2').is_digital, False)
    eq_(a.pin.get('A2').is_analog, True)


def test_name():
    a = ArduinoTree()
    eq_(a.pin.get(8).name, 'D8')
    eq_(a.pin.get('D8').name, 'D8')
    eq_(a.pin.get(15).name, 'A1')
    eq_(a.pin.get('A2').name, 'A2')


def test_dig():
    a = ArduinoTree()
    pin = a.pin.get(8)
    pin.reset()

    pin.write_mode(OUTPUT)

    pin.write_digital_value(1)

    eq_(pin.read_digital_value(), 1)
    eq_(pin.digital_value, 1)

    pin.write_mode(INPUT)
    pin.write_mode(OUTPUT)

    pin.write_digital_value(0)
    eq_(pin.read_digital_value(), 0)
    eq_(pin.digital_value, 0)

    pin.write_mode(INPUT)
    pin.write_mode(OUTPUT)

    pin.write_mode(INPUT)
    pin.write_pullup(True)

    eq_(pin.read_digital_value(), 1)
    eq_(pin.digital_value, 1)


def test_an():
    a = ArduinoTree()
    pin = a.pin.get('A0')
    pin.mode = INPUT
    ok_an(pin.read_analog_value())
    ok_an(pin.analog_value)

    pin.write_pullup(True)

    ok_an(pin.read_analog_value(), pullup=True)


def test_mode():
    a = ArduinoTree()
    DDRB = a.register.get('DDRB')
    pin = a.pin.get(8)
    pin.reset()

    eq_(pin.mode, INPUT)
    eq_(pin.read_mode(), INPUT)
    eq_(DDRB.value, 0)

    pin.write_mode(OUTPUT)
    eq_(pin.mode, OUTPUT)
    eq_(pin.read_mode(), OUTPUT)
    eq_(DDRB.value, 1)

    DDRB.value = 0
    eq_(pin.mode, INPUT)

    pin.mode = OUTPUT
    eq_(pin.mode, OUTPUT)

    pin.reset()
    eq_(pin.mode, INPUT)


def test_pullup():
    a = ArduinoTree()
    pin = a.pin.get(8)
    pin.write_pullup(True)


def test_pin_range():
    a = ArduinoTree()
    eq_(a.pin.count, 20)
    eq_(a.pin.count_analog, 6)
    eq_(a.pin.count_digital, 14)

#     eq_(a.pin.get.range_all, range(0, 20))
#     eq_(a.pin.get.range_analog, range(14, 20))
#     eq_(a.pin.get.range_digital, range(0, 14))

    a.pin.get('A5')
    exc_(ValueError, lambda: a.pin.get('A6'))
    exc_(ValueError, lambda: a.pin.get('D14'))



#!/usr/bin/env python

# Author: Kevin Ng
# Description: read from EEPROM
# Dependencies: None

from nanpy import (EepromLib, SerialManager)
import sys

connection = SerialManager()
e = EepromLib(connection=connection)


def display(a_list):
    for i in range(0, e.size // 16 - 1):
        for j in range(16):
            sys.stdout.write("%02x:" % a_list[j + i * 16])
        print


def read_eeprom():
    data_list = []
    for i in range(0, e.size):
        s = e.read(i)
        data_list.append(s)
    return data_list


if __name__ == "__main__":
    print("Reading eeprom (%s bytes)..." % e.size)
    display(read_eeprom())



from nanpy.arduinoboard import ArduinoObject
from nanpy.arduinoboard import (arduinoobjectmethod, arduinoclassmethod, returns)

class DallasTemperature(ArduinoObject):

    def __init__(self, pin):
        ArduinoObject.__init__(self)
        self.pin = pin
        self.id = self.call('new', pin)

    @arduinoobjectmethod
    def setResolution(self, bits):
        pass

    @returns(int)
    @arduinoobjectmethod
    def getResolution(self, bits):
        pass

    @returns(int)
    @arduinoobjectmethod
    def getDeviceCount(self):
        pass

    @arduinoobjectmethod
    def getAddress(self, index):
        val = self.call('getAddress')
        if val == "1":
            return val
        return val.split(' ')


    @arduinoobjectmethod
    def requestTemperatures(self, address = None):
        pass

    @returns(float)
    @arduinoobjectmethod
    def getTempC(self, address):
        pass

    @returns(float)
    @arduinoobjectmethod
    def getTempF(self, address):
        pass

    @classmethod
    @returns(float)
    @arduinoclassmethod
    def toCelsius(cls, value):
        pass

    @classmethod
    @returns(float)
    @arduinoclassmethod
    def toFahrenheit(cls, value):
        pass




from nanpy.arduino import Arduino
from nose.tools import eq_, ok_
from tests.util import ok_an
import common


def setup():
    common.setup()


def test():
    ok_(Arduino.millis() > 1)

    ok_an(Arduino.analogRead(0))

    Arduino.pinMode(13, 1)
    Arduino.pinMode(13, 0)



from nanpy.arduinotree import ArduinoTree
from nanpy.watchdog import Watchdog
import common


def setup():
    common.setup()


def test():
    a = ArduinoTree(connection=None)
    a.watchdog.reset()
    a.watchdog.enable(a.watchdog.WDTO_1S)
    a.watchdog.disable()


def test2():
    watchdog = Watchdog(connection=None)
    watchdog.reset()
    watchdog.enable(watchdog.WDTO_1S)
    watchdog.disable()



#!/usr/bin/env python

# Author: Andrea Stagi <stagi.andrea@gmail.com>
# Description: get the current time from a ntp server and show it on a lcd
# Dependencies: ntplib (http://pypi.python.org/pypi/ntplib/)

import ntplib
from nanpy import (SerialManager, Lcd)
from datetime import datetime
from time import sleep

connection = SerialManager()

ntp_client = ntplib.NTPClient()
response = ntp_client.request('europe.pool.ntp.org', version=3)

time = int(response.tx_time)

pins = [7, 8, 9, 10, 11, 12]
cols, rows = 16, 2
lcd = Lcd(pins, [cols, rows], connection=connection)

while (1):
    lcd.setCursor(0, 0)
    lcd.printString((datetime.fromtimestamp(time)).strftime('%Y-%m-%d'))
    if time % 2:
        time_format = '%H:%M'
    else:
        time_format = '%H %M'
    lcd.setCursor(0, 1) 
    lcd.printString((datetime.fromtimestamp(time)).strftime(time_format))
    sleep(1)
    time += 1



from nanpy.arduinopin import to_pin_number
from nanpy.counter import Counter
from nose.tools import eq_
from tests.util import exc_


def test_to_pin_number():
    eq_(to_pin_number(0), 0)
    eq_(to_pin_number(1, A0=None), 1)
    eq_(to_pin_number(20, A0=5), 20)

    eq_(to_pin_number('D0', A0=None), 0)
    eq_(to_pin_number('D7', A0=8), 7)
    exc_(ValueError, lambda: to_pin_number('D7', A0=7))
    eq_(to_pin_number('D20', A0=21), 20)

    eq_(to_pin_number('A0', A0=0), 0)
    eq_(to_pin_number('A0', A0=7), 7)
    eq_(to_pin_number('A15', A0=5), 20)
    exc_(ValueError, lambda: to_pin_number('A7', A0=None))

    class Dummy(object):
        pass
    obj = Dummy()
    obj.pin_number = 23
    eq_(to_pin_number(obj, A0=0), 23)


def test_calculate_multiplier():
    Mhz = 1e6

    eq_(Counter.calculate_multiplier(16 * Mhz), 1)
    eq_(Counter.calculate_multiplier(8 * Mhz), 1)
    eq_(Counter.calculate_multiplier(4 * Mhz), 1)
    eq_(Counter.calculate_multiplier(2 * Mhz), 1)
    eq_(Counter.calculate_multiplier(1 * Mhz), 1)

    eq_(Counter.calculate_multiplier(20 * Mhz), 20 / 16.0)
    eq_(Counter.calculate_multiplier(10 * Mhz), 10 / 8.0)



from nanpy.arduinoboard import ArduinoObject
from nanpy.arduinoboard import (arduinoobjectmethod, returns)

class DHT(ArduinoObject):

    DHT11 = 11
    DHT22 = 22
    DHT21 = 21
    AM2301 = 21

    def __init__(self, pin, _type, count = 6):
        ArduinoObject.__init__(self)
        self.id = self.call('new', pin, _type, count)

    @returns(int)
    @arduinoobjectmethod
    def begin(self):
        pass

    @returns(float)
    @arduinoobjectmethod
    def readHumidity(self):
        pass

    @returns(float)
    @arduinoobjectmethod
    def readTemperature(self, value = False):
        pass



import logging
log = logging.getLogger(__name__)


class I2C_CommunicationError(Exception):
    error_texts = {
        # 0: 'success',
        1: 'data too long to fit in transmit buffer',
        2: 'received NACK on transmit of address',
        3: 'received NACK on transmit of data',
        4: 'other error',
    }

    def __init__(self, error_code):
        self.error_code = error_code
        Exception.__init__(self, self.error_texts[error_code])


class I2C_Base(object):

    ''
    bus_initialised = False
    address = None

    def _begin(self):
        if not self.bus_initialised:
            self.wire.begin(self.address)
            self.bus_initialised = True


class I2C_Master(I2C_Base):

    """High level interface for I2C Master."""

    def __init__(self, wire):
        self.wire = wire

    def request(self, address, quantity):
        """Used by the master to request bytes from a slave device.

        :param address: the 7-bit address of the device to request bytes from
        :param quantity: the number of bytes to request
        :param stop: boolean. true will send a stop message after the request, releasing the bus. false will continually send a restart after the request, keeping the connection active.
        :returns: byte : the number of bytes returned from the slave device

        http://arduino.cc/en/Reference/WireRequestFrom

        """
        self._begin()
        n = self.wire.requestFrom(address, quantity, stop=True)
        if n < quantity:
            log.info('slave sent less bytes than requested')
        ls = n * [None]
        for i in range(n):
            ls[i] = self.wire.read()
        return ls

    def send(self, address, data):
        """"""
        self._begin()
        self.wire.beginTransmission(address)
        for b in data:
            self.wire.write(b)
        error_code = self.wire.endTransmission()
        if error_code != 0:
            raise I2C_CommunicationError(error_code)

    def scan(self):
        """The i2c_scanner uses the return value of the Write.endTransmisstion
        to see if a device did acknowledge to the address.

        original source: http://playground.arduino.cc/Main/I2cScanner#.Uxs4Wdt4iJM

        """
        self._begin()
        ls = []
        for address in range(128):
            try:
                self.send(address, [])
                ls.append(address)
            except I2C_CommunicationError:
                pass

        return ls


class I2C_Slave(I2C_Base):

    """High level interface for I2C Slave."""

    def __init__(self, wire, address):
        '''
        :param address: slave address
        '''
        self.wire = wire
        self.address = address

    def receive(self):
        self._begin()
        n = self.wire.available()
        ls = n * [None]
        for i in range(n):
            ls[i] = self.wire.read()
        return ls

    def send(self, data):
        """"""
        self._begin()
        for b in data:
            self.wire.write(b)



import common
from nanpy.arduinotree import ArduinoTree
from nose.tools import eq_
from nose.tools import ok_


def setup():
    common.setup()


def test():
    a = ArduinoTree()
    eq_(a.core.digitalPinToBitMask(2), 4)
    eq_(a.core.digitalPinToPort(2), 4)
    eq_(a.core.digitalPinToTimer(2), 0)
    eq_(a.core.analogInPinToBit(2), 2)


def test_ports():
    a = ArduinoTree()
    eq_(a.core.portInputRegister(0), 0)   # NOT_A_PORT
    eq_(a.core.portInputRegister(1), 0)   # NOT_A_PORT
    eq_(a.core.portInputRegister(2), 35)  # PINB
    eq_(a.core.portInputRegister(3), 38)  # PINC
    eq_(a.core.portInputRegister(4), 41)  # PIND

    eq_(a.core.portModeRegister(0), 0)   # NOT_A_PORT
    eq_(a.core.portModeRegister(1), 0)   # NOT_A_PORT
    eq_(a.core.portModeRegister(2), 36)  # DDRB
    eq_(a.core.portModeRegister(3), 39)  # DDRC
    eq_(a.core.portModeRegister(4), 42)  # DDRD

    eq_(a.core.portOutputRegister(0), 0)   # NOT_A_PORT
    eq_(a.core.portOutputRegister(1), 0)   # NOT_A_PORT
    eq_(a.core.portOutputRegister(2), 37)  # PORTB
    eq_(a.core.portOutputRegister(3), 40)  # PORTC
    eq_(a.core.portOutputRegister(4), 43)  # PORTD



from nanpy.ad9850 import AD9850
from nanpy.arduinotree import ArduinoTree
import time
from nanpy.serialmanager import SerialManager


def measure(a, n, f, root='a'):
    start = time.time()
    for x in range(n):
        cmd = root + '.' + f
        try:
            locals()[root] = a
            exec (cmd)
        except:
            print (cmd)
            raise
    dt = time.time() - start
    print (
        '%-35s %8.2f ms per call, %5.0f call per second' %
        (f, 1000.0 * dt / n, 1.0 / dt * n))


def checkspeed(n):
    connection = SerialManager()
    connection.open()
    a = ArduinoTree(connection=connection)

    print ('performance test for ArduinoTree()')
    print ('n=%s' % n)
    print('')

    measure(a, n, 'api.digitalRead(0)')
    measure(a, n, 'api.digitalWrite(0,0)')
    measure(a, n, 'api.analogRead(0)')
    measure(a, n, 'api.analogWrite(0,0)')
    measure(a, n, 'api.pinMode(0,0)')
    measure(a, n, 'api.millis()')
    measure(a, n, 'api.shiftOut(0, 1, 0, 15)')

    a.define.get('__TIME__')  # init cache
    measure(a, n, 'define.get("__TIME__")')

    a.register.get('DDRB')  # init cache
    measure(a, n, 'register.get("DDRB").read_value()')
    measure(a, n, 'register.get("DDRB").address')
    measure(a, n, 'register.get("DDRB").size')

    a.vcc.read()  # init cache
    measure(a, n, 'vcc.read()')

    measure(a, n, 'pin.get(0).read_digital_value()')
    measure(a, n, 'pin.get(0).read_mode()')

    a.eeprom.size  # init cache
    measure(a, n, 'eeprom.size')
    measure(a, n, 'eeprom.read(0)')

    print('')
    print ('performance test for AD9850()')
    print('')

    ad9850 = AD9850([0, 1, 2, 3])
    measure(ad9850, n, 'setup()', root='ad9850')
    measure(ad9850, n, 'write_frequency(400)', root='ad9850')


if __name__ == '__main__':
    checkspeed(n=100)



"""High frequency PWM mode demo."""

from nanpy.arduinotree import ArduinoTree
from nanpy.serialmanager import SerialManager

FREQ = 10007


def highfreqpwm():
    connection = SerialManager()
    a = ArduinoTree(connection=connection)
    pin9 = a.pin.get(9)
    pin9.mode = 1
    pin9.write_digital_value(1)
    pwm = pin9.pwm
    print ('set  frequency=%s Hz' % FREQ)
    pwm.set_high_freq_around(FREQ)
    print ('real frequency=%s Hz' % pwm.read_frequency())


if __name__ == '__main__':
    highfreqpwm()



#!/usr/bin/env python

# Description: read all RAM

from nanpy.arduinotree import ArduinoTree
from nanpy.serialmanager import SerialManager
import sys


def display(a_list, size):
    for i in range(0, size // 16 - 1):
        for j in range(16):
            sys.stdout.write('%02x:' % a_list[j + i * 16])
        print


def ramdump():
    connection = SerialManager()
    a = ArduinoTree(connection=connection)
    size = a.ram.size
    free = a.ram.free()
    print('Reading RAM (%s bytes, %s bytes free)...' % (size, free))

    data_list = []
    for i in range(0, size):
        s = a.ram.read(i)
        data_list.append(s)

    display(data_list, size)


if __name__ == '__main__':
    ramdump()



from config import config
from nose.tools import eq_, ok_
import common
from nanpy.arduinotree import ArduinoTree
from nanpy.define import DefineFeature


def setup():
    common.setup()


def test_defs():
    a = ArduinoTree()

    eq_(a.define.get('A0'), config['A0'])

    d = a.define.as_dict
    print (d)

    eq_(a.define.get('ARDUINO'), config['ARDUINO'])
    eq_(d['ARDUINO'], config['ARDUINO'])

    eq_(d['A0'], config['A0'])
    eq_(d['F_CPU'], config['F_CPU'])
    eq_(d['MCU'], config['MCU'])
    ok_('xx' not in d)
    ok_(len(d.keys()) > 15, len(d.keys()))

    ARDUINO = a.define.get('ARDUINO')
    ok_(ARDUINO >= 100)
    ok_(ARDUINO < 200)

    NUM_DIGITAL_PINS = a.define.get('NUM_DIGITAL_PINS')
    ok_(NUM_DIGITAL_PINS >= 10)
    ok_(NUM_DIGITAL_PINS < 1000)

    for x in d:
        assert x.strip(), 'empty define:-->%s<--' % x


def test_defs2():
    a = ArduinoTree()
    define = DefineFeature()
    eq_(define.as_dict, a.define.as_dict)



from nanpy.arduinotree import ArduinoTree
from nose.tools import eq_, ok_
from tests.util import ok_an
import common


def setup():
    common.setup()


def test():
    a = ArduinoTree()
    pin13 = a.pin.get(13)
    ok_(a.api.millis() > 1)

    ok_an(a.api.analogRead(0))

    eq_(pin13.read_mode(), 0)
    a.api.pinMode(13, 1)
    eq_(pin13.read_mode(), 1)
    a.api.pinMode(13, 0)
    eq_(pin13.read_mode(), 0)



from nanpy.arduinoboard import ArduinoObject
from nanpy.arduinoboard import (arduinoobjectmethod, returns)

class CapacitiveSensor(ArduinoObject):

    def __init__(self, pin1, pin2):
        ArduinoObject.__init__(self)
        self.id = self.call('new', pin1, pin2)

    @returns(int)
    @arduinoobjectmethod
    def capacitiveSensor(self, samples):
        pass

    @returns(int)
    @arduinoobjectmethod
    def capacitiveSensorRaw(self, samples):
        pass




from __future__ import with_statement
from nanpy.arduinotree import ArduinoTree
from nanpy.counter import Counter
from nose.tools import ok_, eq_
from config import config
import common

OUTPUT = 1


def setup():
    common.setup()


def test_counter():
    a = ArduinoTree()
    p = a.pin.get(5)
    p.write_mode(OUTPUT)
    p.pwm.write_value(128)

    counter1 = a.counter
    counter2 = Counter(F_CPU=config['F_CPU'])

    print ('frequencies_available: %s' % p.pwm.frequencies_available)
    for fset in p.pwm.frequencies_available:
        p.pwm.frequency = fset
        print('---------------------------')
        print('fset=%s' % fset)
        print('---------------------------')
        for ms in [10, 20, 50, 100, 200, 500, 1000]:
            print ('gate=%s ms' % ms)
            t = ms / 1000.0
            error = a.counter.error(t)
            print ('error=%s' % error)

            f1 = counter1.read_frequency(t)
            f2 = counter2.read_frequency(t)
            print (f1, f2)
            diff1 = abs(f1 - fset)
            diff2 = abs(f2 - fset)
            ok_(diff1 <= (1.1 * error))
            ok_(diff2 <= (1.1 * error))



#!/usr/bin/env python

# a simple example to read from a DHT sensor

from nanpy import DHT

# http://learn.adafruit.com/dht/connecting-to-a-dhtxx-sensor
# DHT sensor connected to digital pin 10
dht = DHT(10,DHT.DHT22)

print("temperature is %f degrees Celcius" % dht.readTemperature())
print("temperature is %f degrees Celcius" % dht.readTemperature(False))
print("temperature is %f degrees Fahrenheit" % dht.readTemperature(True))
print("humidity is %f %%" % dht.readHumidity())




from nanpy.serialmanager import serial_manager
from threading import Lock

mutex = Lock()


def _write(data, connection):
    if connection:
        connection.write('%s\0' % str(data))
    else:
        serial_manager.write('%s\0' % str(data))


def return_value(connection):
    if connection:
        return connection.readline().replace('\r\n', '')
    else:
        return serial_manager.readline().replace('\r\n', '')


def _call(namespace, id, args, connection):

    toprint = []
    nel = 0

    mutex.acquire()
    _write(namespace, connection)
    _write(id, connection)

    for arg in args:
        if type(arg) == type(list()):
            for el in arg:
                if el != None:
                    toprint.append(el)
                    nel += 1
        else:
            if arg != None:
                toprint.append(arg)
                nel += 1

    _write(nel - 1, connection)

    for elprint in toprint:
        _write(elprint, connection)

    ret = return_value(connection)
    mutex.release()
    return ret


def returns(fconv):
    def wrapf(func):
        def wrapper(self, *args, **kwargs):
#             try:
                return fconv(func(self, *args, **kwargs))
#             except ValueError:
#                 return -1
        return wrapper
    return wrapf


def arduinoobjectmethod(funct, *args, **kwargs):
    def wrapper(self, *args, **kwargs):
        call_pars = [funct.__name__]
        call_pars.extend(args)
        funct(self, *args, **kwargs)
        connection = self.connection if hasattr(self, 'connection') else None
        return _call(self.namespace, self.id, call_pars, connection=connection)
    return wrapper

# def arduinoclassmethod(funct):
#     def wrapper(cls, *args, **kwargs):
#         call_pars = [funct.__name__]
#         call_pars.extend(args)
#         funct(cls, *args, **kwargs)
#         return _call(cls.__name__, 0, call_pars)
#     return wrapper


def class_get_firmware_id(cls):
    if hasattr(cls, 'get_firmware_id'):
        return cls.get_firmware_id()
    elif hasattr(cls, 'firmware_id'):
        return cls.firmware_id
    else:
        return cls.__name__


def arduinoclassmethod(param):
    is_func = hasattr(param, '__call__')
    if is_func:
        funct = param

        def wrapper(cls, *args, **kwargs):
            cls_name = class_get_firmware_id(cls)
            call_pars = [funct.__name__]
            call_pars.extend(args)
            funct(cls, *args, **kwargs)
            return _call(cls_name, 0, call_pars, connection=None)
        return wrapper
    else:
        name = param

        def wrapper2(funct):
            def wrapper(cls, *args, **kwargs):
                cls_name = class_get_firmware_id(cls)
                call_pars = [name]
                call_pars.extend(args)
                funct(cls, *args, **kwargs)
                return _call(cls_name, 0, call_pars, connection=None)
            return wrapper
        return wrapper2


def _wrapper3(name):
    def wrapper2(funct):
        def wrapper(obj, *args, **kwargs):
            cls_name = class_get_firmware_id(obj.__class__)
            connection = obj.connection if hasattr(obj, 'connection') else None
            call_pars = [name if name else funct.__name__]
            call_pars.extend(args)
            funct(obj, *args, **kwargs)
            return _call(cls_name, 0, call_pars, connection=connection)
        return wrapper
    return wrapper2


def arduinomethod(param):
    is_func = hasattr(param, '__call__')
    if is_func:
        funct = param
        name = None
        return _wrapper3(name)(funct)
    else:
        name = param
        return _wrapper3(name)


class ArduinoObject(object):
    connection=None

    def __init__(self, connection=None):
        self.namespace = self.__class__.__name__
        self.id = 0
        self.connection = connection

    def call(self, *args):
        return _call(self.namespace, self.id, args, self.connection)

    def __del__(self):
        return _call(self.namespace, self.id, ["remove"], self.connection)


class FirmwareClass(object):
    connection = None
    firmware_id = None

    @classmethod
    def get_firmware_id(cls):
        if cls.firmware_id:
            return cls.firmware_id
        else:
            return cls.__name__

    def __init__(self, connection=None):
        self.connection = connection





from __future__ import division
from nanpy.arduinoboard import arduinomethod, returns, FirmwareClass
from nanpy.classinfo import check4firmware
import time


@check4firmware
class CounterLib(FirmwareClass):
    firmware_id = 'Counter'

    @arduinomethod
    def begin(self, gate_time_ms):
        pass

    @arduinomethod
    def end(self):
        pass

    @returns(bool)
    @arduinomethod
    def available(self):
        pass

    @returns(int)
    @arduinomethod
    def read(self):
        pass


class Counter(object):

    """Access to counter.

    Examples:

        Counter.read()

    original lib: http://www.pjrc.com/teensy/td_libs_FreqCount.html

    """
    def __init__(self, connection=None, F_CPU=None):
        self.connection = connection
        self.lib = CounterLib(connection)
        self.multiplier = 1
        if F_CPU:
            self.multiplier = self.calculate_multiplier(F_CPU)

    @classmethod
    def calculate_multiplier(cls, F_CPU):
        multiplier = 1
        # calculate multiplier for non supported frequencies
        if F_CPU >= 16000000:
            multiplier = F_CPU / 16000000
        elif F_CPU >= 8000000:
            multiplier = F_CPU / 8000000
        elif F_CPU >= 4000000:
            multiplier = F_CPU / 4000000
        elif F_CPU >= 2000000:
            multiplier = F_CPU / 2000000
        elif F_CPU >= 1000000:
            multiplier = F_CPU / 1000000
        else:
            multiplier = 1000000 / F_CPU
        return multiplier

    def read(self, gate_time=1):
        """read pulse count under ``gate_time`` seconds."""
        gate_time_ms = int(1000 * gate_time)
        self.lib.begin(gate_time_ms)
        try:
            time.sleep(gate_time * 1.1)
            n = self.lib.read()
        finally:
            self.lib.end()
        return n * self.multiplier

    def read_frequency(self, gate_time=1):
        """read frequency in hertz using ``gate_time`` in seconds."""
        return self.read(gate_time) / gate_time

    def error(self, gate_time=1):
        """frequency measurement error in hertz using ``gate_time`` in
        seconds."""
        return self.multiplier / gate_time



#!/usr/bin/env python

# Author: Andrea Stagi <stagi.andrea@gmail.com>
# Description: just a test for DallasTemperature
# Dependencies: None

from nanpy import DallasTemperature

sensors = DallasTemperature(5)

print("There are %d devices connected on pin %d" % (sensors.getDeviceCount(), sensors.pin))
addr = sensors.getAddress(0)
sensors.setResolution(9)
while True:
    sensors.requestTemperatures(0)
    temp = sensors.getTempC(addr)
    print("The temperature, in Celsius degrees is %0.2f" % temp)
    print("Let's convert it in Fahrenheit degrees: %0.2f" % DallasTemperature.toFahrenheit(temp))




from nanpy.arduinoboard import arduinomethod, returns, FirmwareClass
from nanpy.classinfo import check4firmware
from nanpy.memo import memoized


@check4firmware
class ArduinoCore(FirmwareClass):

    """Access to Arduino functions which are not part of the public API."""

    firmware_id = 'Core'

    @memoized
    @returns(int)
    @arduinomethod
    def digitalPinToBitMask(self, pin):
        """nonpublic Arduino function."""

    @memoized
    @returns(int)
    @arduinomethod
    def digitalPinToPort(self, pin):
        """nonpublic Arduino function."""

    @memoized
    @returns(int)
    @arduinomethod
    def digitalPinToTimer(self, pin):
        """nonpublic Arduino function."""

    @memoized
    @returns(int)
    @arduinomethod
    def analogInPinToBit(self, pin):
        """nonpublic Arduino function."""

    @memoized
    @returns(int)
    @arduinomethod
    def portOutputRegister(self, port):
        """nonpublic Arduino function.

        @return: RAM address

        """

    @memoized
    @returns(int)
    @arduinomethod
    def portInputRegister(self, port):
        """nonpublic Arduino function.

        @return: RAM address

        """

    @memoized
    @returns(int)
    @arduinomethod
    def portModeRegister(self, port):
        """nonpublic Arduino function.

        @return: RAM address

        """



from nanpy.arduinoboard import ArduinoObject
from nanpy.arduinoboard import (arduinoobjectmethod, returns)

class OneWire(ArduinoObject):
    
    def __init__(self, pin):
        ArduinoObject.__init__(self)
        self.id = self.call('new', pin)

    def search(self):
        val = self.call('search')
        if val == "1":
            return val
        return val.split(' ')

    @arduinoobjectmethod
    def select(self, address):
        pass

    @returns(int)
    @arduinoobjectmethod
    def read(self):
        pass

    @arduinoobjectmethod
    def reset_search(self):
        pass

    @arduinoobjectmethod
    def reset(self):
        pass

    @arduinoobjectmethod
    def write(self, address, value=None):
        pass




from nanpy.arduinoboard import (arduinoclassmethod, returns)
from nanpy.memo import memoized

class EEPROM(object):

    @classmethod
    @arduinoclassmethod
    def write(cls, address, value):
        pass

    @classmethod
    @returns(int)
    @arduinoclassmethod
    def read(cls, address):
        pass
    
    @classmethod
    @memoized
    @returns(int)
    @arduinoclassmethod
    def size(cls):
        pass



#!/usr/bin/env python

# Description:

from nanpy.arduinotree import ArduinoTree
from nanpy.i2c import I2C_Master
from nanpy.serialmanager import SerialManager
import logging
log = logging.getLogger(__name__)


def main():
    connection = SerialManager(sleep_after_connect=2)
    connection.open()
    print connection.device
    a = ArduinoTree(connection=connection)
    master = I2C_Master(a.wire)
    print ['0x%02x' % x for x in master.scan()]


if __name__ == '__main__':
    main()



from nanpy.arduinoapi import ArduinoApi
from nanpy.arduinopin import to_pin_number


class ShiftOut(object):
    LSBFIRST, MSBFIRST = 0, 1

    def __init__(self, data_pin, clock_pin,
                 bit_order=0, connection=None, A0=None):
        """Shifts out a byte of data one bit at a time.

        :param data_pin: the pin on which to output each bit (int or string)
        :param clock_pin: the pin to toggle once the dataPin has been set to the correct value (int or string)
        :param bit_order: which order to shift out the bits; either MSBFIRST or LSBFIRST.

        """
        if connection and hasattr(connection, 'shiftOut'):
            self._shiftOut_func = connection.shiftOut
        else:
            api = ArduinoApi(connection=connection)
            self._shiftOut_func = api.shiftOut
        self.bit_order = bit_order
        self.data_pin_number = to_pin_number(data_pin, A0)
        self.clock_pin_number = to_pin_number(clock_pin, A0)

    def write_data(self, data):
        """Shifts out a byte of data one bit at a time.

        :param data: the data to shift out. (byte)

        """
        self._shiftOut_func(
            self.data_pin_number,
            self.clock_pin_number,
            self.bit_order,
            data)



"""Dump all possible values from the AVR."""

from nanpy.arduinotree import ArduinoTree
from nanpy.serialmanager import SerialManager
from pprint import pprint
import inspect
from nanpy.serialmanager import SerialManager

FORMAT = '%-20s = %20s'


def dump(obj, selected_names=None):
    if selected_names:
        ls = selected_names
    else:
        ls = dir(obj)
    for attr in ls:
        if not attr.startswith('__'):
            if not inspect.ismethod(getattr(obj, attr)):
                    print(FORMAT % (attr, getattr(obj, attr)))


def dump_dict(d):
    for defname in sorted(d.keys()):
        defvalue = d[defname]
        print(FORMAT % (defname, defvalue))


def dumpall():
    connection = SerialManager()
    a = ArduinoTree(connection=connection)

    print((FORMAT + ' V') % ('read_vcc', a.vcc.read()))
    print((FORMAT + ' sec') % ('millis', a.api.millis() / 1000.0))

    print('')
    print('================================')
    print('firmware classes:')
    print('================================')
    print('status:')
    pprint(a.connection.classinfo.firmware_class_status)
    print('unknown ids:')
    pprint(a.connection.classinfo.unknown_firmware_ids)

    print('')
    print('================================')
    print('pins:')
    print('================================')

    print(FORMAT % ('total_pin_count', a.pin.count))
    print(FORMAT % ('digital_names', a.pin.names_digital))
    print(FORMAT % ('analog_names', a.pin.names_analog))

    for pin_number in range(a.pin.count):
        print('---------- pin_number=%s ---------------' % pin_number)
        pin = a.pin.get(pin_number)
        dump(
            pin,
            'name pin_number pin_number_analog is_digital is_analog avr_pin mode digital_value analog_value programming_function'.split())
        if pin.pwm.available:
            print('--- pwm ---')
            dump(pin.pwm, '''frequency frequencies_available base_divisor divisor divisors_available
                                timer_mode
                                timer_register_name_a
                                timer_register_name_b
                                wgm
            '''.split())

    print('')
    print('================================')
    print('defines:')
    print('================================')
    dump_dict(a.define.as_dict)

    print('')
    print('================================')
    print('registers:')
    print('================================')
    for x in a.register.names:
        r = a.register.get(x)
        if r.size == 2:
            v = '0x%04X' % (r.value)
        else:
            v = '  0x%02X' % (r.value)

        print('%-20s = %s @0x%2X (size:%s)' % (r.name, v, r.address, r.size))


if __name__ == '__main__':
    dumpall()



from nanpy.memo import memoized
from nose.tools import eq_


class TestClass(object):
    i = 1

    @memoized
    def met(self, x):
        TestClass.i += 1
        return TestClass.i


def test1():
    o = TestClass()
    eq_(o.i, 1)
    o.met(3)
    eq_(o.i, 2)
    o.met(3)
    eq_(o.i, 2)
    o.met(4)
    eq_(o.i, 3)
    o.met(4)
    o.met(4)
    o.met(3)
    o.met(3)
    o.met(4)
    eq_(o.i, 3)

    o2 = TestClass()
    eq_(o.i, 3)
    o2.met(4)
    o2.met(4)
    o2.met(4)
    o2.met(4)
    eq_(o.i, 4)



from nanpy.fwinfo import libc_version, arduino_version, avr_name, libc_date, \
    compile_datetime, firmware_info
from nose.tools import eq_
import datetime


def test():
    eq_(libc_version(10604), '1.6.4')
    eq_(libc_version('10604'), '1.6.4')
    eq_(libc_version(11111), '1.11.11')

    eq_(libc_date(20081106), datetime.date(2008, 11, 6))
    eq_(libc_date('20081106'), datetime.date(2008, 11, 6))

    eq_(arduino_version(105), '1.0.5')
    eq_(arduino_version('105'), '1.0.5')

    eq_(avr_name('__AVR_ATmega328P__'), 'ATmega328P')

    eq_(compile_datetime('Mar 28 2008', '17:40:12'),
        datetime.datetime(2008, 3, 28, 17, 40, 12))

    eq_(firmware_info(dict(
                      MCU='__AVR_ATmega328P__',
                      ARDUINO='105',
                      __DATE__='Mar 28 2014',
                      __TIME__='17:40:12',
                      __VERSION__='4.3.2',
                      __AVR_LIBC_DATE_='20081106',
                      __AVR_LIBC_VERSION__='10604',

                      )),
        {'gcc_version': '4.3.2',
         'libc_version': '1.6.4',
                         'libc_date': datetime.date(2008, 11, 6),
                         'arduino_version': '1.0.5',
                         'avr_name': 'ATmega328P',
                         'compile_datetime':
         datetime.datetime(2014, 3, 28, 17, 40, 12),
         }
        )



from nanpy.arduinoboard import arduinomethod, returns, FirmwareClass
from nanpy.classinfo import check4firmware
from nanpy.memo import memoized


@check4firmware
class RegisterArray(FirmwareClass):
    firmware_id = 'R'

    @memoized
    @returns(int)
    @arduinomethod('c')
    def count(self):
        pass

    @returns(int)
    @arduinomethod('r')
    def read(self, index):
        pass

    @arduinomethod('w')
    def write(self, index, value):
        pass

    @memoized
    @returns(int)
    @arduinomethod('a')
    def address(self, index):
        pass

    @memoized
    @returns(int)
    @arduinomethod('s')
    def size(self, index):
        pass

    @memoized
    @arduinomethod('n')
    def name(self, index):
        pass


class Register(object):

    """Representation of an AVR register."""

    def __init__(self, index, arr):
        """"""
        self._arr = arr
        self.index = index

    def read_value(self):
        """Read register value.

        Examples:

            a=ArduinoTree()
            a.register.get('DDRB').read_value()

        """
        return self._arr.read(self.index)

    def write_value(self, value):
        """Write register value.

        Examples:

            a=ArduinoTree()
            a.register.get('DDRB').write_value(1)

        """
        return self._arr.write(self.index, value)

    value = property(read_value, write_value)

    @property
    def name(self):
        """Register name.
        """
        return self._arr.name(self.index)

    @property
    def address(self):
        """Register address.

        Examples:

            a=ArduinoTree()
            a.register.get('DDRB').address

        """
        return self._arr.address(self.index)

    @property
    def size(self):
        """Register size in bytes. 1 or 2.

        Examples:

            a=ArduinoTree()
            a.register.get('DDRB').size

        """
        return self._arr.size(self.index)


class RegisterProxy(object):

    """Proxy object to allow easy register access.

        Examples:

            a=ArduinoTree()
            a.register.proxy.DDRB = 1
            print( a.register.proxy.DDRB )

    """

    def __init__(self, arr, name_index_dict):
        self._name_index_dict = name_index_dict
        self._arr = arr

    def __getattr__(self, name):
        if name in ['_name_index_dict', '_arr']:
            return object.__getattribute__(self, name)

        index = self._name_index_dict[name]
        return self._arr.read(index)

    def __setattr__(self, name, value):
        if name in ['_name_index_dict', '_arr']:
            self.__dict__[name] = value
            return

        index = self._name_index_dict[name]
        return self._arr.write(index, value)


class RegisterFeature(object):

    def __init__(self, connection=None):
        self._arr = RegisterArray(connection)

    @property
    @memoized
    def name_index_dict(self):
        d = dict()
        for i in range(self._arr.count()):
            d[self._arr.name(i)] = i
        assert len(d)
        return d

    @property
    def names(self):
        """Get all register names as a sorted list."""
        return sorted(self.name_index_dict.keys())

    @property
    @memoized
    def count(self):
        """Get register count."""
        return len(self.name_index_dict.keys())

    @memoized
    def get(self, name):
        try:
            index = self.name_index_dict[name]
        except KeyError:
            raise ValueError('Unknown register: %s' % name)
        return Register(index, self._arr)

    @property
    @memoized
    def proxy(self):
        return RegisterProxy(self._arr, self.name_index_dict)



from nanpy.arduinoboard import arduinomethod, returns, FirmwareClass
from nanpy.classinfo import check4firmware
from nanpy.memo import memoized


def auto_convert(x):
    try:
        return int(x)
    except ValueError:
        return x


@check4firmware
class DefineArray(FirmwareClass):
    firmware_id = 'D'

    @property
    @memoized
    @returns(int)
    @arduinomethod('c')
    def count(self):
        pass

    @memoized
    @arduinomethod('n')
    def name(self, index):
        pass

    @memoized
    @returns(auto_convert)
    @arduinomethod('v')
    def value(self, index):
        pass


class DefineFeature(object):

    """Access to firmware constants.

    Examples:

        a=ArduinoTree()
        a.define.get('F_CPU') # AVR frequency
        a.define.get('__DATE__') # firmware upload date

    """

    def __init__(self, connection=None):
        self._arr = DefineArray(connection)

    @property
    @memoized
    def as_dict(self):
        """return all constants and their values in a dictionary."""
        d = dict()
        for i in range(self._arr.count):
            d[self._arr.name(i)] = self._arr.value(i)
        assert len(d)
        return d

    @property
    def names(self):
        return sorted(self.as_dict.keys())

    @property
    @memoized
    def count(self):
        return len(self.as_dict)

    def get(self, name):
        """get a constant.

        Examples:

            a=ArduinoTree()
            a.define.get('F_CPU') # AVR frequency

        """
        count = self._arr.count
        assert count
        for i in range(count):
            n = self._arr.name(i)
            if n == name:
                return self._arr.value(i)
        raise ValueError('% was not found!' % name)



import time


class Vcc(object):

    def __init__(self, register):
        self.register = register

    def read(self):
        """VCC in Volt. This works on Arduinos with a 328 or 168 only.

        Source: http://code.google.com/p/tinkerit/wiki/SecretVoltmeter

        How it works

        The Arduino 328 and 168 have a built in precision voltage reference of 1.1V.
        This is used sometimes for precision measurement,
        although for Arduino it usually makes more sense to measure against Vcc,
        the positive power rail.
        The chip has an internal switch that selects which pin the analogue to digital converter reads.
        That switch has a few leftover connections, so the chip designers wired them up to useful signals.
        One of those signals is that 1.1V reference.
        So if you measure how large the known 1.1V reference is in comparison to Vcc,
        you can back-calculate what Vcc is with a little algebra. That is how this works.

        """

        reg = self.register.proxy
        # 0b01001110 = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1)
        reg.ADMUX = 0x4E
        # Wait for Vref to settle
        time.sleep(0.002)
        # Convert
        reg.ADCSRA |= 0x40  # 0b01000000 = _BV(ADSC)
        while reg.ADCSRA & 0x40:  # bit_is_set(ADCSRA, ADSC)
            time.sleep(0.001)
        result = reg.ADCL | (reg.ADCH << 8)
        bandgap_voltage = 1.1  # Volt
        result = (bandgap_voltage * 1023) / result
        return result



from __future__ import division
import logging

log = logging.getLogger(__name__)


class PwmError(Exception):
    pass


class BiDict():

    def __init__(self, dic):
        self.norm = dic
        self.inv = dict([(v, k) for k, v in dic.items()])

base_divisor = {
    3: 512,
    5: 256,
    6: 256,
    9: 512,
    10: 512,
    11: 512,
}


_div1 = BiDict({
               #                0: None,
               1: 1,
               2: 8,
               3: 64,
               4: 256,
               5: 1024,
               #                6: None,
               #                7: None,
               })
_div2 = BiDict({
               #                0: None,
               1: 1,
               2: 8,
               3: 32,
               4: 64,
               5: 128,
               6: 256,
               7: 1024,
               })
divisor_mapping = {
    3: _div2,
    5: _div1,
    6: _div1,
    9: _div1,
    10: _div1,
    11: _div2,
}

TIMERS_A = ['NOT_ON_TIMER',
            'TCCR0A',
            'TCCR0A',
            'TCCR1A',
            'TCCR1A',
            None,  # TODO: atmega8
            'TCCR2A',
            'TCCR2A',
            ]

TIMERS_B = ['NOT_ON_TIMER',
            'TCCR0B',
            'TCCR0B',
            'TCCR1B',
            'TCCR1B',
            'TCCR2',
            'TCCR2B',
            'TCCR2B',
            ]

timer_mask = 7  # 0b111

# TODO: pwm_mode  read/write
# TODO: read mappings


class ArduinoPwmPin(object):

    '''Object-oriented representation of the pin PWM functionality
    '''
    DEFAULT_DIVISOR = 64

    def __init__(self, nr, define, register, core, api):
        self.nr = nr
        self.register = register
        self.F_CPU = define.get('F_CPU')
        self.api = api
        self.core = core

    def reset(self):
        '''reset to the PWM default state: default frequency is set
        '''
        if not self.available:
            return
        self.write_divisor(self.DEFAULT_DIVISOR)

    @property
    def available(self):
        """PWM is available for this pin?"""
        timer_id = self._timer_id()
        return timer_id > 0 and timer_id < len(TIMERS_B)
#        return self.nr in timer_register

    def _check(self):
        if not self.available:
            raise PwmError('pwm not available for pin: %s' % self.nr)

    def write_value(self, value):
        """same as analogWrite."""
        self._check()
#         assert self.mcu.pins.read_mode(self.nr) == OUTPUT
        self.api.analogWrite(self.nr, value)

    @property
    def divisors_available(self):
        """list of available divisors."""
        try:
            return list(divisor_mapping[self.nr].norm.values())
        except KeyError:
            return []

    def read_divisor(self):
        """read current divisor."""
        self._check()
        d = divisor_mapping[self.nr]
        return d.norm[self.read_timer_mode()]

    def write_divisor(self, value):
        """write current divisor."""
        self._check()
        d = divisor_mapping[self.nr]
        self.write_timer_mode(d.inv[value])
    divisor = property(read_divisor, write_divisor)

    def _timer_id(self):
        return self.core.digitalPinToTimer(self.nr)

    def _timer_register_name(self, variant='B'):
        self._check()
        i = self._timer_id()
        return dict(A=TIMERS_A, B=TIMERS_B)[variant][i]

    @property
    def timer_register_name_a(self):
        return self._timer_register_name(variant='A')

    @property
    def timer_register_name_b(self):
        return self._timer_register_name(variant='B')

    def read_timer_mode(self):
        reg_name = self.timer_register_name_b
        return self.register.get(reg_name).read_value() & timer_mask

    def write_timer_mode(self, value):
        assert value <= 7
        reg_name = self.timer_register_name_b
        reg = self.register.get(reg_name)
        old = reg.value & ~timer_mask
        reg.value = (old | value)

    timer_mode = property(read_timer_mode, write_timer_mode)

    @property
    def base_divisor(self):
        self._check()
        return base_divisor[self.nr]

    def calculate_frequency(self, divisor):
        return 1.0 * self.F_CPU / self.base_divisor / divisor

    @property
    def frequencies_available(self):
        """list of available frequencies."""
        ls = sorted([self.calculate_frequency(x)
                    for x in self.divisors_available])
        return ls

    def read_frequency(self):
        self._check()
        wgm = self.read_wgm()
        if wgm == 14:
            # high freq mode
            return self.F_CPU / self.register.get('ICR1').value
        else:
            return self.calculate_frequency(self.read_divisor())

    def write_frequency(self, value):
        self._check()
        d = divisor_mapping[self.nr]
        for x in self.divisors_available:
            f = self.calculate_frequency(x)
            if abs(f - value) <= 1:
                self.write_timer_mode(d.inv[x])
                return
    frequency = property(read_frequency, write_frequency)

    def read_wgm(self):
        """read waveform generation mode."""
        self._check()
        rega = self.timer_register_name_a
        regb = self.timer_register_name_b
        if regb == 'TCCR1B':
            maskb = 24  # 0b00011000
        else:
            maskb = 8  # 0b00001000
        maska = 3  # 0b00000011
        a = self.register.get(rega).value & maska
        b = self.register.get(regb).value & maskb
        return a + (b >> 1)

    wgm = property(read_wgm, None)

    def _check_high_freq(self):
        # TODO: read config
        if self.nr not in [9, 10]:
            raise PwmError('high freq pwm not available for pin: %s' % self.nr)

    def set_high_freq_around_pwm(self, top, fill):
        'F_CPU/divisor'
        # TODO:
        d = top
        self._check_high_freq()
        assert d >= 2

        self.write_divisor(1)
        self.write_value(128)

        TCCR1A = self.register.get('TCCR1A')
        TCCR1B = self.register.get('TCCR1B')
        TCNT1 = self.register.get('TCNT1')
        ICR1 = self.register.get('ICR1')
        OCR1A = self.register.get('OCR1A')
        OCR1B = self.register.get('OCR1B')

        TCCR1A.value = 2 + \
            (240 & TCCR1A.value)  # 0b00000010 + (0b11110000 & reg.TCCR1A)
        TCCR1B.value = 25  # 0b00011001

        TCNT1.value = 0
        ICR1.value = d
        OCR1A.value = fill
        OCR1B.value = fill

    def set_high_freq_around(self, freq):
        """set high frequency mode, and try to set frequency as close as
        possible.

        available frequencies:  F_CPU/N (N=2..65535)

        Example: F_CPU=16000000 (default) -> frequencies=244Hz-8MHz

        TODO: test it on more AVRs, read config from firmware

        """
        top = int(self.F_CPU / freq + 0.5)
        assert 1 < top < (1 << 16)
        self.set_high_freq_around_pwm(top, int(top / 2))



from nanpy.arduinotree import ArduinoTree
from nanpy.serialmanager import SerialManager
import time


def boot_time():
    connection = SerialManager()
    a = ArduinoTree(connection=connection)
    a.watchdog.enable(a.watchdog.WDTO_500MS)
    time.sleep(1)
    a.connection.flush_input()
    x = a.api.millis()
    return 1000 - 500 - x


if __name__ == '__main__':
    print ('boot_time %s ms' % boot_time())



from __future__ import division
from nanpy.arduinotree import ArduinoTree
from nanpy.memo import memoized
from nanpy.shiftout import ShiftOut


LOW, HIGH = 0, 1
OUTPUT = 1


class AD9850(object):

    """controllling the AD9850 DDS module.

    code is based on this info: http://nr8o.dhlpilotcentral.com/?p=83

    """
    clock = 125000000  # note 125 MHz clock on 9850
    _frequency = 0

    def __init__(self, pin_names, connection=None):
        '''
        :param pin_names: [W_CLK, FQ_UD, DATA, RESET]
        W_CLK      // Pin 8 - connect to AD9850 module word load clock pin (CLK)
        FQ_UD        // Pin 9 - connect to freq update pin (FQ)
        DATA        // Pin 10 - connect to serial data load pin (DATA)
        RESET       // Pin 11 - connect to reset pin (RST).
        '''
        self.connection = connection
        self.pin_names = dict(
            W_CLK=pin_names[0],
            FQ_UD=pin_names[1],
            DATA=pin_names[2],
            RESET=pin_names[3])

    @property
    @memoized
    def arduino(self):
        return ArduinoTree(connection=self.connection)

    def create_pin_object(self, name):
        return self.arduino.pin.get(self.pin_names[name])

    @property
    @memoized
    def W_CLK(self):
        return self.create_pin_object('W_CLK')

    @property
    @memoized
    def FQ_UD(self):
        return self.create_pin_object('FQ_UD')

    @property
    @memoized
    def DATA(self):
        return self.create_pin_object('DATA')

    @property
    @memoized
    def RESET(self):
        return self.create_pin_object('RESET')

    @property
    @memoized
    def shift(self):
        return ShiftOut(self.DATA, self.W_CLK, 0, connection=self.connection)

    def pulse_high(self, pin):
        pin.write_digital_value(HIGH)
        pin.write_digital_value(LOW)

    def setup(self):
        self.FQ_UD.write_mode(OUTPUT)
        self.W_CLK.write_mode(OUTPUT)
        self.DATA.write_mode(OUTPUT)
        self.RESET.write_mode(OUTPUT)

        self.pulse_high(self.RESET)
        self.pulse_high(self.W_CLK)
        # this pulse enables serial mode - Datasheet page 12 figure 10
        self.pulse_high(self.FQ_UD)

    def tfr_byte(self, value):
        self.shift.write_data(value)

    def write_frequency(self, frequency):
        # frequency calc from datasheet page 8 =
        # <sys clock> * <frequency tuning word>/2^32
        CONST = 4294967295  # 2^32 - 1
        freq = frequency * CONST / self.clock
        freq = int(freq)
        for b in range(4):
            self.tfr_byte((freq >> (8 * b)) & 0xFF)

        self.tfr_byte(0x000)  # Final control byte, all 0 for 9850 chip
        self.pulse_high(self.FQ_UD)  # Done!  Should see output
        self._frequency = frequency

    # frequency read is not implemented
    frequency = property(lambda self: self._frequency, write_frequency)



from nanpy.arduinotree import ArduinoTree
from nanpy.serialmanager import serial_manager
from nose.tools import ok_
import tempfile


def exc_(excClass, callableObj, *args, **kwargs):
    """Fail unless an exception of class excClass is thrown by callableObj when
    invoked with arguments args and keyword arguments kwargs.

    If a different type of exception is thrown, it will not be caught,
    and the test case will be deemed to have suffered an error, exactly
    as for an unexpected exception.

    """
    try:
        callableObj(*args, **kwargs)
    except excClass:
        return
    else:
        if hasattr(excClass, '__name__'):
            excName = excClass.__name__
        else:
            excName = str(excClass)
        raise Exception('%s not raised' % excName)


def ok_an(x, pullup=False):
    print (x)
    ok_(x in range(1024))


def ok_vcc(vcc):
    print ('vcc=', vcc)
    ok_(vcc < 5.5)
    ok_(vcc > 3)


def tmpdir(dir=None, suffix=''):
    x = tempfile.mkdtemp(suffix=suffix, prefix='nanpy_test_', dir=dir)
    return x



from nanpy.arduinoboard import arduinoclassmethod, returns, FirmwareClass
import time


class Arduino(FirmwareClass):
    firmware_id = 'A'

    LOW, HIGH = 0, 1
    INPUT, OUTPUT = 0, 1

    @classmethod
    @arduinoclassmethod('dw')
    def digitalWrite(cls, pin, value):
        pass

    @classmethod
    @returns(int)
    @arduinoclassmethod('r')
    def digitalRead(cls, pin):
        pass

    @classmethod
    @arduinoclassmethod('aw')
    def analogWrite(cls, pin, value):
        pass

    @classmethod
    @returns(int)
    @arduinoclassmethod('a')
    def analogRead(cls, pin):
        pass

    @classmethod
    @arduinoclassmethod('pm')
    def pinMode(cls, pin, mode):
        pass

    @classmethod
    def delay(cls, value):
        time.sleep(float(value)/1000)
        
    @classmethod
    @returns(int)
    @arduinoclassmethod('m')
    def millis(cls):
        pass

    @classmethod
    def map(cls, x, in_min, in_max, out_min, out_max):
        return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min




class NullConnection(object):

    def write(self, value):
        print('sending:%s' % repr(value))

    def readline(self):
        return ''

    def flush_input(self):
        """Flush input buffer, discarding all it's contents."""



"""soft reset demo."""

from nanpy.arduinotree import ArduinoTree
from nanpy.serialmanager import SerialManager


def print_millis(a):
    print ('uptime: %s sec' % (a.api.millis() / 1000.0))


def reset_demo():
    connection = SerialManager()
    a = ArduinoTree(connection=connection)
    print_millis(a)
    print ('soft reset')
    a.soft_reset()
    print_millis(a)

if __name__ == '__main__':
    reset_demo()



from nanpy.arduinoboard import ArduinoObject
from nanpy.arduinoboard import arduinoobjectmethod

class Stepper(ArduinoObject):

    def __init__(self, revsteps, pin1, pin2, speed=None):
        ArduinoObject.__init__(self)
        self.id = self.call('new', revsteps, pin1, pin2)
        if speed:
            self.setSpeed(speed)

    @arduinoobjectmethod
    def setSpeed(self, value):
        pass

    @arduinoobjectmethod
    def step(self, value):
        pass



from __future__ import division
from nanpy.arduinoapi import ArduinoApi
from nanpy.arduinocore import ArduinoCore
from nanpy.arduinopin import PinFeature
from nanpy.classinfo import ClassInfo
from nanpy.counter import Counter
from nanpy.define import DefineFeature
from nanpy.eepromobj import EepromLib
from nanpy.fwinfo import firmware_info
from nanpy.memo import memoized
from nanpy.ram import RAM
from nanpy.register import RegisterFeature
from nanpy.serialmanager import SerialManager, serial_manager
from nanpy.vcc import Vcc
from nanpy.watchdog import Watchdog
from nanpy.wire import Wire
import time


class ArduinoTree(object):

    """Object tree model of an Arduino.

    Examples:

        a=ArduinoTree()
        a.pin.get(9).read_digital_value()

    """

    def __init__(self, connection=None):
        ''
        self.connection = connection
        if not connection:
            self.connection = serial_manager
        if not hasattr(self.connection, 'classinfo'):
            self.connection.classinfo = ClassInfo(self.connection)

    @property
    @memoized
    def api(self):
        """Access to Arduino API."""
        return ArduinoApi(self.connection)

    @property
    @memoized
    def pin(self):
        """Object-oriented representation of an Arduino pin"""
        return PinFeature(self.define, self.register, self.core, self.ram, self.api)

    @property
    @memoized
    def define(self):
        """Access to firmware constants."""
        return DefineFeature(self.connection)

    @property
    @memoized
    def register(self):
        """Direct access to AVR registers."""
        return RegisterFeature(self.connection)

    @property
    @memoized
    def watchdog(self):
        """Direct access to watchdog timer."""
        return Watchdog(self.connection)

    @property
    @memoized
    def eeprom(self):
        """Access to EEPROM."""
        return EepromLib(self.connection)

    @property
    @memoized
    def ram(self):
        """Access to RAM."""
        return RAM(self.connection)

    @property
    @memoized
    def counter(self):
        """Access to counter."""
        return Counter(self.connection, F_CPU=self.define.get('F_CPU'))

    @property
    @memoized
    def core(self):
        """Access to Arduino functions which are not part of the public API."""
        return ArduinoCore(self.connection)

    @property
    @memoized
    def vcc(self):
        """Access to VCC."""
        return Vcc(self.register)

    def soft_reset(self):
        """Resets the AVR, the registers will be reset to their known, default
        settings.

        Details: http://www.nongnu.org/avr-libc/user-manual/FAQ.html#faq_softreset

        """
        self.watchdog.enable(self.watchdog.WDTO_60MS)
        time.sleep(0.2)

        # TODO: after restart the first read is not correct (why?)
        # This command helps cleaning the read buffer
        self.connection.flush_input()

    @property
    @memoized
    def wire(self):
        """Access to Wire."""
        return Wire(self.connection)

    @property
    @memoized
    def firmware_info(self):
        """"""
        return firmware_info(self.define.as_dict)



from nanpy.arduinotree import ArduinoTree
from nose.tools import eq_
import common


def setup():
    common.setup()


def test():
    a = ArduinoTree()

    eq_(a.eeprom.size, 1024)
    eq_(a.eeprom.read(17), 255)



from config import config
from nanpy.arduinotree import ArduinoTree
from nanpy.register import RegisterFeature
from nanpy.vcc import Vcc
from nose.tools import eq_, ok_
from tests.util import ok_vcc
import common


def setup():
    common.setup()


def test_vcc():
    a = ArduinoTree()
    ok_vcc(a.vcc.read())


def test_vcc2():
    vcc = Vcc(RegisterFeature())
    ok_vcc(vcc.read())



#!/usr/bin/env python

from setuptools import setup, find_packages

setup(name="nanpy",
      version="0.9.1",
      description="Use your Arduino board with Python",
      license="MIT",
      author="Andrea Stagi",
      author_email="stagi.andrea@gmail.com",
      url="http://github.com/nanpy/nanpy",
      packages = find_packages(),
      keywords= "arduino library prototype",
      install_requires=[
        "pyserial",
      ],
      zip_safe = True)



#!/usr/bin/env python

# Author: Andrea Stagi <stagi.andrea@gmail.com>
# Description: a fake DallasTemperature library written in Python
# Dependencies: None

from nanpy import (Arduino, OneWire)
import time

class FakeDallasTemperature():

    def __init__(self, pin):
        self.__ds = OneWire(pin)
        self.__data = 0

    def __fetchAddress(self):
        ds_address = self.__ds.search()

        if ds_address == "1":
            return False

        self.__ds.reset()
        self.__ds.select(ds_address)
        self.__ds.write(0x44, 1)
        Arduino.delay(700)
        present = self.__ds.reset()
        self.__ds.select(ds_address)
        self.__ds.write(0xBE)
        return True

    def __fetchData(self):

        data = []

        for i in range(9):
            val = self.__ds.read()
            data.append(val)

        raw = (data[1] << 8) | data[0]

        type = 1

        if type == 0:

            raw = raw << 3;
            if data[7] == 0x10:
                raw = (raw & 0xFFF0) + 12 - data[6]

        else:
            cfg = (data[4] & 0x60)

            if cfg == 0x00:
                raw = raw << 3
            elif cfg == 0x20:
                raw = raw << 2
            elif cfg == 0x40:
                raw = raw << 1
            else:
                pass

        self.__data = raw

    def getCelsius(self):
        if self.__fetchAddress():
            self.__fetchData()
        return self.__data / 16.0

    def getFahrenheit(self):
        if self.__fetchAddress():
            self.__fetchData()
        return self.getCelsius() * 1.8 + 32.0


from nanpy import Lcd

if __name__ == "__main__":

    lcd = Lcd([7, 8, 9, 10, 11, 12], [16, 2])
    lcd.printString("Loc. London")

    temp_int = FakeDallasTemperature(6)
    temp_ext = FakeDallasTemperature(5)

    while(1):
        lcd.setCursor(0, 1)
        lcd.printString("Ex.%0.0f\xDFC  In.%0.0f\xDFC" % (temp_int.getCelsius(), temp_ext.getCelsius()))




import functools

'''
http://wiki.python.org/moin/PythonDecoratorLibrary
'''


class memoized(object):
    """Decorator that caches a function's return value each time it is called.
    If called later with the same arguments, the cached value is returned, and
    not re-evaluated.
    """
    def __init__(self, func):
        self.func = func
        self.cache = {}

    def __call__(self, *args):
        try:
            return self.cache[args]
        except KeyError:
            value = self.func(*args)
            self.cache[args] = value
            return value
        except TypeError:
            # uncachable -- for instance, passing a list as an argument.
            # Better to not cache than to blow up entirely.
            return self.func(*args)

    def __repr__(self):
        """Return the function's docstring."""
        return self.func.__doc__

    def __get__(self, obj, objtype):
        """Support instance methods."""
        return functools.partial(self.__call__, obj)



#    sleep_after_connect=2 is required for most of the boards !


# custom board with 20MHz quartz and ATMEGA328P
# config = dict(
#     A0=14,
#     ARDUINO=105,
#     F_CPU=20000000.0,
#     MCU='__AVR_ATmega328P__',
#     avr_name='ATmega328P',
#     sleep_after_connect=0,
# )

# custom board with 16MHz quartz and ATMEGA328P
config = dict(
    A0=14,
    ARDUINO=105,
    F_CPU=16000000.0,
    MCU='__AVR_ATmega328P__',
    avr_name='ATmega328P',
    sleep_after_connect=0,
)


# Uno (not tested)
# config = dict(
#     A0=14,
#     ARDUINO=105,
#     F_CPU=16000000.0,
#     MCU='__AVR_ATmega328__',
#     avr_name='ATmega328',
#     sleep_after_connect=2,
# )



from nanpy.arduinotree import ArduinoTree
import common


def setup():
    common.setup()

# set it to 1000 for a good test (4 minutes)
REPEAT_TEST = 10


def test_long():
    a = ArduinoTree()
    for x in range(REPEAT_TEST):
        print (x, ('uptime: %s sec' % (a.api.millis() / 1000.0)))
        a.soft_reset()



from __future__ import with_statement
from config import config
from nanpy.arduinotree import ArduinoTree
from nanpy.shiftout import ShiftOut
from nose.tools import ok_
from nose.tools import eq_
import common


A0 = config['A0']


def setup():
    common.setup()


def test_shift1():
    s = ShiftOut('D13', 'D12', 0, A0=A0)
    s.write_data(0b01010101)


def test_shift2():
    s = ShiftOut(13, 'A1', 0, A0=A0)
    s.write_data(0b01010101)


def test_shift3():
    a = ArduinoTree()
    p = a.pin.get(13)
    s = ShiftOut(p, 12, 0, A0=A0)
    s.write_data(0b01010101)



from nanpy.arduinoboard import ArduinoObject
from nanpy.arduinoboard import arduinoobjectmethod


class Lcd_I2C(ArduinoObject):

    def __init__(self, pins, begin, connection=None):
        '''
        :param pins: [lcd_Addr, enable, Rw, Rs, d4, d5, d6, d7, backlighPin, pol]
        :param begin: [cols, rows]
        '''
        ArduinoObject.__init__(self, connection=connection)
        self.id = self.call('new', pins, begin)

    @arduinoobjectmethod
    def printString(self, value, col=None, row=None):
        pass

    @arduinoobjectmethod
    def setCursor(self, col, row):
        pass

    @arduinoobjectmethod
    def setBacklight(self, value):
        pass



from nanpy.arduinoboard import arduinomethod, returns, FirmwareClass
from nanpy.classinfo import check4firmware
from nanpy.memo import memoized


@check4firmware
class EepromLib(FirmwareClass):
    firmware_id = 'EEPROM'

    @arduinomethod
    def write(self, address, value):
        pass

    @returns(int)
    @arduinomethod
    def read(self, address):
        pass

    @property
    @memoized
    @returns(int)
    @arduinomethod
    def size(self):
        pass



from nanpy.arduinoboard import arduinomethod, returns, FirmwareClass
from nanpy.classinfo import check4firmware


@check4firmware
class Wire(FirmwareClass):

    """access to Wire library
    http://arduino.cc/en/reference/wire
    """
    firmware_id = 'Wire'

    @arduinomethod
    def begin(self, address=None):
        """Initiate the Wire library and join the I2C bus as a master or slave.
        This should normally be called only once.

        :param address: the 7-bit slave address (optional); if not specified,
            join the bus as a master.
        :returns: None

        http://arduino.cc/en/Reference/WireBegin

        """

    @returns(int)
    @arduinomethod
    def requestFrom(self, address, quantity, stop=True):
        """Used by the master to request bytes from a slave device. The bytes
        may then be retrieved with the available() and read() functions.

        :param address: the 7-bit address of the device to request bytes from
        :param quantity: the number of bytes to request
        :param stop: boolean. true will send a stop message after the request,
            releasing the bus. false will continually send a restart after the request,
            keeping the connection active.
        :returns: byte : the number of bytes returned from the slave device

        http://arduino.cc/en/Reference/WireRequestFrom

        """

    @arduinomethod
    def beginTransmission(self, address):
        """Begin a transmission to the I2C slave device with the given address.
        Subsequently, queue bytes for transmission with the write() function
        and transmit them by calling endTransmission().

        :param address: the 7-bit address of the device to transmit to
        :returns: None

        http://arduino.cc/en/Reference/WireBeginTransmission

        """

    @returns(int)
    @arduinomethod
    def endTransmission(self, stop=True):
        """Ends a transmission to a slave device that was begun by
        beginTransmission() and transmits the bytes that were queued by
        write().

        :param stop: boolean. true will send a stop message,
            releasing the bus after transmission. false will send a restart,
            keeping the connection active.
        :returns: byte, which indicates the status of the transmission:
            0:success
            1:data too long to fit in transmit buffer
            2:received NACK on transmit of address
            3:received NACK on transmit of data
            4:other error

        http://arduino.cc/en/Reference/WireEndTransmission

        """

    @returns(int)
    @arduinomethod
    def write(self, value):
        """Writes data from a slave device in response to a request from a
        master, or queues bytes for transmission from a master to slave device.

        (in-between calls to beginTransmission() and endTransmission()).

        :param value: a value to send as a single byte
        :returns: byte: write() will return the number of bytes written

        http://arduino.cc/en/Reference/WireWrite

        """

    @returns(int)
    @arduinomethod
    def available(self, pin, value):
        """Returns the number of bytes available for retrieval with read().
        This should be called on a master device after a call to requestFrom()
        or on a slave inside the onReceive() handler.

        :returns: The number of bytes available for reading.

        http://arduino.cc/en/Reference/WireAvailable

        """

    @returns(int)
    @arduinomethod
    def read(self, pin, value):
        """Reads a byte that was transmitted from a slave device to a master
        after a call to requestFrom() or was transmitted from a master to a
        slave.

        :returns: The next byte received

        http://arduino.cc/en/Reference/WireRead

        """



#!/usr/bin/env python

# Author: Andrea Stagi <stagi.andrea@gmail.com>
# Description: keeps your led blinking
# Dependencies: None

from nanpy import (ArduinoApi, SerialManager)
from time import sleep

connection = SerialManager()
a = ArduinoApi(connection=connection)

a.pinMode(13, a.OUTPUT)

for i in range(10000):
    a.digitalWrite(13, (i + 1) % 2)
    sleep(0.2)




import datetime


def avr_name(MCU):
    """The name of the AVR.

    Example: __AVR_ATmega328P__ -> ATmega328P

    """
    return MCU.strip('_').split('_')[-1]


def arduino_version(ARDUINO):
    """
    Example: 105  -> 1.0.5

    """
    return '.'.join(str(ARDUINO))


def compile_datetime(DATE, TIME):
    '''
    Example: Mar 28 2014, 17:40:12
    -> datetime.datetime(2008, 3, 28, 17, 40, 12)
    '''
    dt = datetime.datetime.strptime(
        DATE + ' ' + TIME,
        '%b %d %Y %H:%M:%S')
    return dt


def libc_version(AVR_LIBC_VERSION):
    """
    Example: 10604  -> 1.6.4

    """
    s = str(AVR_LIBC_VERSION)
    ls = [s[0], s[1:3], s[3:5]]
    ls = map(int, ls)
    ls = map(str, ls)
    return '.'.join(ls)


def libc_date(AVR_LIBC_DATE):
    """
    Example: 20081106  -> 2008.11.06

    """
    s = str(AVR_LIBC_DATE)
    ls = [s[0:4], s[4:6], s[6:8]]
    ls = map(int, ls)
    d = datetime.date(*ls)
    return d


def firmware_info(define_dict):
    d = dict()
    d['avr_name'] = avr_name(define_dict['MCU'])
    d['arduino_version'] = arduino_version(define_dict['ARDUINO'])
    d['compile_datetime'] = compile_datetime(define_dict['__DATE__'],
                                             define_dict['__TIME__'],
                                             )
    d['gcc_version'] = define_dict['__VERSION__']
    d['libc_version'] = libc_version(define_dict['__AVR_LIBC_VERSION__'])
    d['libc_date'] = libc_date(define_dict['__AVR_LIBC_DATE_'])

    return d
